# import tensorboard # just for launching TensorBoard Session on VSCode
from typing import Tuple, Dict, Any
from tqdm import tqdm
from termcolor import colored
from loguru import logger
import optuna
from config.config_definition import config as config_orig
from .utils.misc import grab_gpu, parse_args
from .utils.logging import create_log_folders, ContexualLogger
from .utils.hyperparameter_tuning import run_optuna, suggest_parameters
from .utils.checkpoint import initialize_objectives, finalize_objectives
from .utils.training import prepare_for_training, run_one_epoch


def objective(
    trial: optuna.Trial, device: str, config_orig: Dict[str, Any]
) -> Tuple[float, ...]:
    """
    Defines the objective function for Optuna to optimize.

    Args:
    - trial: An optuna Trial object.
    - device: The device on which to run the computation.
    - config_orig: A dictionary containing model configuration parameters.

    Returns:
    - Any: The value of the objective function.
    """
    # copy and tune for each trial
    config = config_orig.copy()

    # Suggest parameters if the experimental phase is "tuning"
    # Run params sanity check under other phases.
    # This function may modify config in-place.
    suggest_parameters(trial, config)

    # This function modifies config
    create_log_folders(config)

    # Outputs under this context will be logged into one file.
    # you shall not use @logger.catch other than the functions used under
    # this context because it may let the code running even under errors.
    # The log file will be investigated earch time the code exits from
    # the context. Error will be raised when error logs are found.
    with ContexualLogger(config, is_stop_at_error=False):
        model, optimizer, data_loaders, tb_writer = prepare_for_training(config, device)

        best = initialize_objectives(config)
        global_step = 0
        for epoch in tqdm(
            range(config["NUM_EPOCHS"]),
            mininterval=5,
            desc=colored("Epoch progress: ", "blue"),
            colour="blue",
        ):
            logger.info(f"Starting epoch #{epoch}.")
            best, global_step = run_one_epoch(
                trial,
                model,
                optimizer,
                device,
                data_loaders,
                tb_writer,
                config,
                best,
                global_step,
            )

    return finalize_objectives(best)


def main() -> None:
    """
    The main function of the program.

    Returns:
    - None
    """
    # use arguments to overwrite the config file
    parse_args(config_orig)
    # grab a GPU
    device = grab_gpu(config_orig)
    # start learning!
    run_optuna(objective, device, config_orig)

    return


if __name__ == "__main__":
    main()



import sys

from .models.losses import *
from .models.optimizers import *
from .models.temporal_integrators import *

from .utils.checkpoint import *
from .utils.data_processing import *
from .utils.hyperparameter_tuning import *
from .utils.logging import *
from .utils.training import *
from .utils.performance_metrics import *
from .utils.misc import *



from setuptools import setup, find_packages
from os import path

# Get the long description from the README file
with open(
    path.join(path.abspath(path.dirname(__file__)), "README.md"), encoding="utf-8"
) as f:
    long_description = f.read()

setup(
    name="SPRT-TANDEM",
    version="0.1.2",
    license="MIT",
    description="SPRT-TANDEM for sequential density ratio estimation to simultaneously optimize speed and accuracy of early-classification.",
    author="Akinori F. Ebihara",
    author_email="aebihara@nec.com",
    packages=find_packages(),
    long_description=long_description,
    long_description_content_type="text/markdown",
    include_package_data=True,
    url="https://github.com/Akinori-F-Ebihara/SPRT-TANDEM-PyTorch",
    keywords=[
        "Sequential Probability Ratio Test" "likelihood ratio",
        "density ratio estimation",
        "early classification",
        "artificial intelligence",
        "machine learning",
    ],
    install_requires=["torch", "torchinfo", "optuna", "loguru"],
    python_requires=">=3.8",
)
